module Autocomplete.Styling exposing (View(Menu, List, Item, SelectedItem, Input), Classes)

{-| Styling module for the Autocomplete component.

The autocomplete consists of a menu, a list, list items, and an input.
This module includes functions to provide css class names for styling those
child views.

Styling is easy as:
```
testData : List String
testData =
  [ "elm"
  , "makes"
  , "coding"
  , "life"
  , "easy"
  ]

getClasses : Styling.View -> Styling.Classes
getClasses view =
  case view of
    Styling.Menu ->
      [ ( "autocomplete-menu", True ) ]

    Styling.List ->
      [ ( "autocomplete-list", True ) ]

    Styling.Item ->
      [ ( "autocomplete-item", True ) ]

    Styling.SelectedItem ->
      [ ( "autocomplete-selected-item", True ) ]

    Styling.Input ->
      [ ( "autocomplete-input", True ) ]


main : Program Never
main =
  let
    config =
      Autocomplete.Config.defaultConfig
        |> Autocomplete.Config.setClassesFn getClasses
  in
    Html.beginnerProgram
      { model = initWithConfig testData config
      , update = (\act model -> fst (update act model))
      , view = view
      }
```

# Child Views
@docs View

# Definition
@docs Classes

-}


{-| A list of class names and their associated status (added/removed) as a boolean value.
-}
type alias Classes =
  List ( String, Bool )


{-| The stylable views of the Autocomplete component.
-}
type View
  = Menu
  | List
  | Item
  | SelectedItem
  | Input
